import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Stack;

public class DFS {
    Graph graph;
    
    /** Play around with graphs */
    public static void main(String[] args) {
        DFS dfs= new DFS();
        dfs.buildGraph();
        
        dfs.graph.printGraph();
        dfs.graph.printVisited();
        
        
        int start= 3; // node to start a graph traversal
        System.out.println("Doing DFS");
        dfs.DFS(start);
        System.out.println();
        dfs.graph.printVisited();
        
        dfs.graph.clearVisited();
        dfs.graph.printVisited();
        System.out.println("Doing DFS1");
        dfs.DFS1(start);
        System.out.println();
        dfs.graph.printVisited();
        
        dfs.graph.clearVisited();
        dfs.graph.printVisited();
        System.out.println("Doing DFSIterative");
        dfs.DFSIterative(start);
        System.out.println();
        dfs.graph.printVisited();

    }
    
    /** Build a graph and store it in game. */
    public void buildGraph() {
        Pair[] edges= new Pair[] {
                new Pair(0, 2), new Pair(0, 4), new Pair(0, 5),
                new Pair(1, 2), new Pair(1, 3), new Pair(1, 4),
                new Pair(2, 1), new Pair(2, 4), new Pair(2, 5),
                new Pair(3, 5),
                new Pair(4, 1), new Pair(4, 5),
                new Pair(5, 3)
        };
        graph= new Graph(6, edges);
        
    }
    
    /** Visit every node that is reachable along
     * a path of unvisited nodes starting at node n.
     * Precondition: n is not visited.*/
    public void DFS(int n) {
       graph.visited[n]= true;
       ArrayList<Integer> nes= graph.getNeighbors(n);
       for (Integer ne : nes) {
           if (!graph.visited[ne]) DFS(ne);
       }
    }
    

    /** Visit every node that is reachable along
     * a path of unvisited nodes starting at node n.
    */
    public void DFS1(int n) {
      if (graph.visited[n]) return;
      graph.visited[n]= true;
      ArrayList<Integer> nes= graph.getNeighbors(n);
      for (Integer ne : nes) {
          DFS1(ne);
      }
    }
    
    /** Visit every node that is reachable along
     * a path of unvisited nodes starting at node n.
    */
    public void DFSIterative(int n) {
        Stack<Integer> st= new Stack<Integer>();
        st.push(n);
        // inv: Every node initially reachable from n along
        // a path of unvisited nodes is reachable along suc
        // a path from some node in st.
        while (!st.isEmpty()) {
            int node= st.pop();
            if (!graph.visited[node]) {
                graph.visited[node]= true;
                ArrayList<Integer> nes= graph.getNeighbors(node);
                for (Integer ne : nes) {
                    st.add(ne);
                }
            }
        }
    }
    
    /** Visit every node that is reachable along
     * a path of unvisited nodes starting at node n
     * in breadth-first order. */
    public void BFS(int n) {
        LinkedList<Integer> queue= new LinkedList<Integer>();
        queue.add(n);
        // inv: Every node initially reachable from n along
        // a path of unvisited nodes is reachable along such
        // a path from some node in st. 
        // Unvisited nodes at level i from n occur before unvisited
        //  nodes at level i+1 from n
        while (!queue.isEmpty()) {
            int node= queue.remove(0);
            if (!graph.visited[node]) {
                graph.visited[node]= true;
                ArrayList<Integer> nes= graph.getNeighbors(node);
                for (Integer ne : nes) {
                    queue.add(ne);
                }
            }
        }
    }

}
