import java.util.ArrayList;
import java.util.Iterator;

/** An instance is a graph maintained as an adjacency matrix */ 
public class Graph {
    public boolean[][] matrix; // adjacency matrix
    public boolean[] visited;  // visited[i] is true iff node i has been visited
    
    public int nodeSize;       // number of nodes
    public int edgeSize;       // number of edges

    /** A graph with n nodes numbers 0..n-1 and edges
     * given by edges. All nodes are unvisited. */
    public Graph(int n, Pair[] edges) {
        nodeSize= n;
        matrix= new boolean[n][n];
        visited= new boolean[n];
        for (Pair p : edges) {
            matrix[p.one][p.two]= true;
        }
    }
    
   /** Return a list of neighbors of n.
    * Precondition: 0 <= n < number of nodes */
    public ArrayList<Integer> getNeighbors(int n) {
        ArrayList<Integer> s= new ArrayList<Integer>();
        for (int k= 0; k < matrix.length; k= k+1) {
            if (matrix[n][k]) s.add(k);
        }
        return s;
    }
    
    /** Visit node n. Precondition: 0 <= n < nodeSize. */
    public void visit(int n) {
        visited[n]= true;
    }
    
    /** clear visited --make all nodes unvisited. */
    public void clearVisited() {
        for (int k= 0; k < visited.length; k= k+1) {
            visited[k]= false;
        }
    }
    
    /** Print the graph.
     * Precondition: the graph has < 100 nodes */
    public void printGraph() {
        System.out.println("Adjacency matrix");
        
        // print heading
        String heading= "     ";
        for (int c= 0; c < matrix.length; c= c+1) {
            heading= heading + make3(c) + " ";
        }
        System.out.println(heading);
        
        // Print graph
        for (int r= 0; r < matrix.length; r= r+1) {
            // print row r
            String row= make3(r) + "  ";
            for (int c= 0; c < matrix.length; c= c+1) {
                row= row + (matrix[r][c] ? "  1" : "  0") + " ";
            }
            System.out.println(row);
        }
        
    }
    
    /** = n in at least 3 chars, with leading blanks added if necessary. */
    public static String make3(int n) {
        if (0 <= n  &&  n < 10) return "  " + n;
        if (0 <= n  &&  n < 100) return " " + n;
        if (-10 < n  &&  n < 0)  return " " + n;
        return "" + n;
    }
    
    /** Print which nodes were visited. */
    public void printVisited() {
        String v= "visited nodes: ";
        for (int n= 0; n < visited.length; n= n+1) {
            if (visited[n]) v= v + " " + n;
        }
        System.out.println(v);
    }

}

